// AttilaBars.h
/////////////////////////////////////////////////////////////////////////////
// This is part of the Attila library.
// Copyright (c) 1999, Attila contributors (see ReadMe.htm).
// All rights reserved. No warranties extended.
/////////////////////////////////////////////////////////////////////////////
// History:
//  -11/11/99:
//   Joe Mariadassou [JMariadassou@syssol.com.au] pointed out a Unicode
//   code having to do w/ sizeof.
//
//  -11/9/99
//   Marshall, Mike [MarshallMJ@intercim.com] added the ability to specify
//   a mask color for the bitmap on the info bar.
//
//  -11/5/99:
//   Marshall, Mike [MarshallMJ@intercim.com] added an ellipse to text that
//   was too small to show in the InfoBar.

#pragma once
#ifndef INC_ATTILABARS
#define INC_ATTILABARS

#include "AttilaCommon.H"
#include "AttilaTranslate.h"

namespace Attila
{

#define GET_X_LPARAM(lp)   ((int)(short)LOWORD(lp))
#define GET_Y_LPARAM(lp)   ((int)(short)HIWORD(lp))

/////////////////////////////////////////////////////////////////////////////
// Menubar
// This is a modified (ATLized) version of the Paul Dilascia's most excellent
// MSJ cool menubar class

// We need to store the object pointer in some thread based data so that
// we can access the object methods in the HookProc.
// This is *exactly during* the duration for which the popup Menu is being
// tracked.

DWORD __declspec(selectany) g_TlsSlot = 0;

template <typename TBase = CToolBarCtrl, typename TWinTraits = CMenuBarTraits>
class CMenuBarImplT 
	: public CWindowImpl<CMenuBarImplT,TBase,TWinTraits>,
	  public CMsgTranslator
{
public:
	DECLARE_WND_SUPERCLASS(_T("Attila_MenuBarBase"),TBase::GetWndClassName());
	
	typedef CMenuBarImplT<TBase,TWinTraits >			thisClass;
	typedef CWindowImpl<CMenuBarImplT,TBase,TWinTraits>	baseClass;

public:	
	BEGIN_MSG_MAP(thisClass)
		MESSAGE_HANDLER(WM_LBUTTONDOWN, OnLButtonDown)				
		MESSAGE_HANDLER(WM_MOUSEMOVE, OnMouseMove)
	END_MSG_MAP()
	
public:
	CMenuBarImplT() :	m_hMenu(0),
						m_bEscapeWasPressed(FALSE),
						m_nPopupTracking(FALSE),
						m_nNewPopup(-1),
						m_hMenuTracking(0),
						m_eTrackingState(TRACK_NONE)
	{
		m_ptMouse.x = 0;
		m_ptMouse.y = 0;
	}

	~CMenuBarImplT() 
	{
		if(m_hMenu)
		{
			::DestroyMenu(m_hMenu);
			m_hMenu = 0;
		}
		
		if(g_TlsSlot)
		{
			TlsFree(g_TlsSlot);
			g_TlsSlot=0;
		}
	}

public:
	HWND Create(	HWND hWndParent, RECT& rcPos,
					UINT nID = 0,			//menubar related
					LPCTSTR szWindowName = NULL,
					DWORD dwStyle = 0, DWORD dwExStyle = 0,
					LPVOID lpCreateParam = NULL)
	{
		if (GetWndClassInfo().m_lpszOrigName == NULL)
			GetWndClassInfo().m_lpszOrigName = GetWndClassName();
		ATOM atom = GetWndClassInfo().Register(&m_pfnSuperWindowProc);

		dwStyle = GetWndStyle(dwStyle);
		dwExStyle = GetWndExStyle(dwExStyle);

		HWND hWnd = CWindowImplBaseT< TBase, TWinTraits >::Create(	hWndParent, 
																	rcPos, szWindowName,
																	dwStyle, dwExStyle, 
																	nID, atom, lpCreateParam);
		ATLASSERT(::IsWindow(hWnd));
		SetButtonStructSize(sizeof(TBBUTTON));

		if(!m_hMenu)
			LoadMenu(nID);
		
		//Assign the top level menu items as toolbar buttons
		AssignMenuButtons();
		
		//AutoSizeit
		AutoSize();
		
		return hWnd;
	}
public:
	enum TRACKINGSTATE // menubar has three states:
	{ 
		TRACK_NONE = 0,   // * normal, not tracking anything
		TRACK_BUTTON,     // * tracking buttons (F10/Alt mode)
		TRACK_POPUP       // * tracking popups
	};

public://msg handlers
	
	LRESULT OnLButtonDown(UINT,WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;
		POINT pt;
		pt.x = LOWORD(lParam);
		pt.y = HIWORD(lParam);
		int iButton = SendMessage(TB_HITTEST,0,(LPARAM)&pt);	
		if (iButton >= 0 && iButton<GetButtonCount())			// if mouse is over a button:
		{
			bHandled = TRUE;
			TrackPopup(iButton);								 //   track it
		}

		return 0L;
	}
	LRESULT OnMouseMove(UINT,WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		POINT pt;pt.x = LOWORD(lParam);pt.y = HIWORD(lParam);

		if (m_eTrackingState==TRACK_BUTTON) 
		{

			// In button-tracking state, ignore mouse-over to non-button area.
			// Normally, the toolbar would de-select the hot item in this case.
			// 
			// Only change the hot item if the mouse has actually moved.
			// This is necessary to avoid a bug where the user moves to a different
			// button from the one the mouse is over, and presses arrow-down to get
			// the menu, then Esc to cancel it. Without this code, the button will
			// jump to wherever the mouse is--not right.

			bHandled = TRUE;
			int iHot = SendMessage(TB_HITTEST,0,(LPARAM)&pt);
			if (IsValidButton(iHot) && !(pt.x == m_ptMouse.x && pt.y == m_ptMouse.y) )
				SetHotItem(iHot);
			return 0L;			 // don't let toolbar get it
		}

		m_ptMouse = pt;			// remember point
		bHandled = FALSE;
		return 0L;
	}

	// JR - Updated signature to match CMsgTranslator
	BOOL Translate(const MSG& rMsg,HACCEL hAccel)
	{
		UINT msg = rMsg.message;

		if (WM_LBUTTONDOWN <= msg && msg <= WM_MOUSELAST) 
		{
			if (rMsg.hwnd != m_hWnd && m_eTrackingState > 0) 
			{
				// user clicked outside menu bar: exit tracking mode
				SetTrackingState(TRACK_NONE);
			}

		} 
		else if( msg == WM_SYSKEYDOWN || msg == WM_SYSKEYUP || msg == WM_KEYDOWN )  
		{
			BOOL bAlt = HIWORD(rMsg.lParam) & KF_ALTDOWN; // Alt key down
			TCHAR vkey = rMsg.wParam;							  // get virt key

			if (vkey==VK_MENU ||(vkey==VK_F10 && !((GetKeyState(VK_SHIFT) & 0x80000000) ||
								   (GetKeyState(VK_CONTROL) & 0x80000000) || bAlt))) 
			{

				// key is VK_MENU or F10 with no alt/ctrl/shift: toggle menu mode
				if (msg==WM_SYSKEYUP) 
				{
					ToggleTrackButtonMode();
				}
				return TRUE;

			} 
			else if( msg == WM_SYSKEYDOWN || msg == WM_KEYDOWN ) 
			{
				if (m_eTrackingState == TRACK_BUTTON) 
				{
					// I am tracking: handle left/right/up/down/space/Esc
					switch (vkey) 
					{
					case VK_LEFT:
					case VK_RIGHT:
						// left or right-arrow: change hot button if tracking buttons
						SetHotItem(GetNextOrPrevButton(GetHotItem(), vkey==VK_LEFT));
						return TRUE;

					case VK_SPACE:  // (personally, I like SPACE to enter menu too)
					case VK_UP:
					case VK_DOWN:
						// MW - Post a simulated arrow-down into the message stream
						// so TrackPopupMenu will read it and move to the first item
						PostMessage(WM_KEYDOWN, VK_DOWN, 1);

						// up or down-arrow: move into current menu, if any
						TrackPopup(GetHotItem());
						return TRUE;

					case VK_ESCAPE:
						// escape key: exit tracking mode
						SetTrackingState(TRACK_NONE);
						return TRUE;
					}
				}

				// Handle alphanumeric key: invoke menu. Note that Alt-X
				// chars come through as WM_SYSKEYDOWN, plain X as WM_KEYDOWN.
				if ((bAlt || m_eTrackingState == TRACK_BUTTON) && ::IsCharAlphaNumeric(vkey)) 
				{
					// Alt-X, or else X while in tracking mode
					UINT nID = 0;
					if (SendMessage(TB_MAPACCELERATOR, (WPARAM)vkey, (LPARAM)&nID)) 
					{
						TrackPopup(nID);	 // found menu mnemonic: track it
						return TRUE;		 // handled
					}
					else if (m_eTrackingState==TRACK_BUTTON && !bAlt) 
					{
						MessageBeep(0);
						return TRUE;
					}
				}

				// Default for any key not handled so far: return to no-menu state
				if (m_eTrackingState > 0) 
				{
					SetTrackingState(TRACK_NONE);
				}
			}
		}
		return FALSE; // not handled, pass along
	}

private:
	void LoadMenu(UINT nMenuID)
	{
		if(!m_hMenu)
			m_hMenu = ::LoadMenu(_Module.GetResourceInstance(), MAKEINTRESOURCE(nMenuID));
	}

	void AssignMenuButtons()
	{
		UINT nMenuItems = m_hMenu ? ::GetMenuItemCount(m_hMenu) : 0;
		int nCount = GetButtonCount();
		while (nCount--) {	DeleteButton(0); }
		
		SetImageList(0); //*absolutely* essential. Try removing it :-)
		for (UINT i=0; i < nMenuItems; i++) 
		{
			TCHAR name[64];
			memset(name, 0, sizeof(name)); // guarantees double-0 at end
			if (::GetMenuString(m_hMenu, i, name, 64, MF_BYPOSITION)) 
			{
				TBBUTTON tbb;
				memset(&tbb, 0, sizeof(tbb));

				tbb.idCommand = ::GetMenuItemID(m_hMenu, i);
				AddStrings(name);
				
				tbb.iString = i;
				tbb.fsState = TBSTATE_ENABLED;
				tbb.fsStyle = TBSTYLE_BUTTON|TBSTYLE_AUTOSIZE;
				tbb.iBitmap = -1;
				tbb.idCommand = i;
				AddButtons(1, &tbb);
			}
		}
	}

	void SetTrackingState(TRACKINGSTATE eState, int nButton = -1)
	{
		if (eState != m_eTrackingState) 
		{
			if (eState == TRACK_NONE)
			{
				nButton = -1;
			}
			SetHotItem(nButton);					 // could be none (-1)

			if (eState == TRACK_POPUP) 
			{
				// set related state stuff
				m_bEscapeWasPressed = FALSE;	 // assume Esc key not pressed
				m_nPopupTracking = nButton;	 // which popup I'm tracking
			}
			m_eTrackingState = eState;
		}
	}
	void ToggleTrackButtonMode()
	{
		if (m_eTrackingState == TRACK_NONE || m_eTrackingState == TRACK_BUTTON) 
		{
			SetTrackingState(m_eTrackingState == TRACK_NONE ? TRACK_BUTTON : TRACK_NONE, 0);
		}
	}

	int GetNextOrPrevButton(int nButton, BOOL bPrev)
	{
		if (bPrev) 
		{
			nButton--;
			if (nButton < 0)
				nButton = GetButtonCount() - 1;
		} 
		else 
		{
			nButton++;
			if (nButton >= GetButtonCount())
				nButton = 0;
		}
		return nButton;
	}

	void TrackPopup(int iButton)
	{
		int nMenuItems = ::GetMenuItemCount(m_hMenu);

		while (iButton >= 0) // while user selects another menu
		{					 
			m_nNewPopup = -1;						 // assume quit after this
			PressButton(iButton, TRUE);		 // press the button
			UpdateWindow();						 // and force repaint now

			// post a simulated arrow-down into the message stream
			// so TrackPopupMenu will read it and move to the first item

			// MW - Do NOT simulate key down and key up; this causes the
			//      first menu item to be selected which is different from
			//      normal behavior. Simulated key strokes are handled during
			//      key and mouse input.
//			PostMessage(WM_KEYDOWN, VK_DOWN, 1);
//			PostMessage(WM_KEYUP, VK_DOWN, 1);

			SetTrackingState(TRACK_POPUP, iButton); // enter tracking state

			// Need to install a hook to trap menu input in order to make
			// left/right-arrow keys and "hot" mouse tracking work.
			
			//First, Grab the TLS slot
			if(g_TlsSlot)
			{
				TlsFree(g_TlsSlot); 
				g_TlsSlot = 0;
			}
			g_TlsSlot = TlsAlloc();
			TlsSetValue(g_TlsSlot,this);
		
			m_hMsgHook = SetWindowsHookEx(WH_MSGFILTER,thisClass::MenuInputFilter, NULL, ::GetCurrentThreadId());

			// get submenu and display it beneath button
			RECT rcButton;	GetRect(iButton, &rcButton);
			ClientToScreen(&rcButton);
			POINT pt  = { rcButton.left, rcButton.top };
			//POINT pt = ComputeMenuTrackPoint(rcButton, tpm);
			HMENU hMenuPopup = ::GetSubMenu(m_hMenu, iButton);
			
			BOOL bRet = TrackPopupMenuEx(hMenuPopup,
				TPM_LEFTALIGN|TPM_LEFTBUTTON|TPM_VERTICAL,
				pt.x, pt.y + 20, m_hWnd,NULL);

			// uninstall hook.
			::UnhookWindowsHookEx(m_hMsgHook);
			m_hMsgHook = NULL;
			
			//Free the TlsSlot
			if(g_TlsSlot)
			{
				TlsFree(g_TlsSlot); 
				g_TlsSlot = 0;
			}


			PressButton(iButton, FALSE);	 // un-press button
			UpdateWindow();					 // and force repaint now

			// If the user exited the menu loop by pressing Escape,
			// return to track-button state; otherwise normal non-tracking state.
			SetTrackingState(m_bEscapeWasPressed ?TRACK_BUTTON : TRACK_NONE, iButton);

			// If the user moved mouse to a new top-level popup (eg from File to
			// Edit button), I will have posted a WM_CANCELMODE to quit
			// the first popup, and set m_nNewPopup to the new menu to show.
			// Otherwise, m_nNewPopup will be -1 as set above.
			// So just set iButton to the next popup menu and keep looping...
			iButton = m_nNewPopup;
		}
	}
	
	BOOL IsValidButton(int nButton) const
	{ 
		return 0 <= nButton && nButton < GetButtonCount(); 
	}

	// Handle menu input event: Look for left/right to change popup menu,
	// mouse movement over over a different menu button for "hot" popup effect.
	// Returns TRUE if message handled (to eat it).
	BOOL OnMenuInput(MSG& m)
	{
		int msg = m.message;

		if (msg==WM_KEYDOWN) 
		{
			// handle left/right-arow.
			WORD vkey = LOWORD(m.wParam);
			if ( (vkey == VK_LEFT )||(vkey == VK_RIGHT) ) 
 			{
				CancelMenuAndTrackNewOne( GetNextOrPrevButton(m_nPopupTracking, vkey == VK_LEFT));

				// MW - Simulate key down to select first item
				PostMessage(WM_KEYDOWN, VK_DOWN, 1);
				return TRUE; // eat it
			}
			else if (vkey == VK_ESCAPE) 
			{
				m_bEscapeWasPressed = TRUE;	 // (menu will abort itself)
			}

		} else if (msg==WM_MOUSEMOVE || msg==WM_LBUTTONDOWN) {
			// handle mouse move or click
			POINT pt ;pt.x = GET_X_LPARAM(m.lParam);pt.y = GET_Y_LPARAM(m.lParam);
			ScreenToClient(&pt);

			if (msg == WM_MOUSEMOVE) 
			{
				if (!(pt.x == m_ptMouse.x && pt.y == m_ptMouse.y))
				{
					int iButton = SendMessage(TB_HITTEST,0,(LPARAM)&pt);
					if (IsValidButton(iButton) && iButton != m_nPopupTracking) 
					{
						// user moved mouse over a different button: track its popup
						CancelMenuAndTrackNewOne(iButton);
					}
					m_ptMouse = pt;
				}

			} 
			else if (msg == WM_LBUTTONDOWN) 
			{
				if (SendMessage(TB_HITTEST,0,(LPARAM)&pt) == m_nPopupTracking) 
				{
					// user clicked on same button I am tracking: cancel menu
					CancelMenuAndTrackNewOne(-1);
					return TRUE; // eat it
				}
			}
		}
		return FALSE; // not handled
	}

	void CancelMenuAndTrackNewOne(int iNewPopup)
	{
		if (iNewPopup != m_nPopupTracking) 
		{
			PostMessage(WM_CANCELMODE);					 // quit menu loop
			m_nNewPopup = iNewPopup;					 // go to this popup (-1 = quit)
		}
	}
	
	static LRESULT CALLBACK MenuInputFilter(int code, WPARAM wp, LPARAM lp)
	{
		MSG *pMsg = (LPMSG)lp;
		thisClass *pMenuBar = (thisClass *)TlsGetValue(g_TlsSlot);
		ATLASSERT(pMenuBar); //hope & pray
		
		if(pMsg)
		{
			return (code==MSGF_MENU && pMenuBar->OnMenuInput(*pMsg)) ? 
				                         TRUE: CallNextHookEx(pMenuBar->m_hMsgHook, code, wp, lp);
		}
		return CallNextHookEx(pMenuBar->m_hMsgHook, code, wp, lp);
	}

	TRACKINGSTATE GetTrackingState(int& iPopup) 
	{
		iPopup = m_nPopupTracking; 
		return m_eTrackingState;
	}

protected:
	HMENU					m_hMenu;
	// menu tracking stuff:
	BOOL					m_bEscapeWasPressed;				// user pressed escape to exit menu
	int						m_nPopupTracking;					// which popup I'm tracking if any
	int						m_nNewPopup;						// next menu to track
	POINT					m_ptMouse;							// mouse location when tracking popup
	HMENU					m_hMenuTracking;					// current popup I'm tracking
	TRACKINGSTATE			m_eTrackingState;					// current tracking state
	HHOOK					m_hMsgHook;
};

typedef CMenuBarImplT<CToolBarCtrl, CMenuBarTraits> CMenuBar;

/////////////////////////////////////////////////////////////////////////////
// Our Toolbar

const int nToolbarID	= 100;
const int nToolbarBMPID = 120;//IDR_MAINFRAME eventually and should be from resource

template <typename TBase = CToolBarCtrl, typename TWinTraits = CToolBarWinTraits>
class CToolBarImplT : public CWindowImpl<CToolBarImplT, TBase, TWinTraits>
{
public:
	DECLARE_WND_SUPERCLASS(_T("Attila_ToolBarBase"),TBase::GetWndClassName());
	
	typedef CToolBarImplT<TBase,TWinTraits>							thisClass;
	typedef CWindowImpl<CToolBarImplT,TBase,TWinTraits>	baseClass;
	
	BEGIN_MSG_MAP(thisClass)
		NOTIFY_CODE_HANDLER(TTN_NEEDTEXT, OnToolTipText)
	END_MSG_MAP()

	LRESULT OnToolTipText(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
	{
    bHandled = false;
		DWORD dwStyle = 0;

		// Ignore this message if tooltips are not enabled
		if( !(GetWndStyle(dwStyle) & TBSTYLE_TOOLTIPS) )
			return 0;

		if( pnmh && idCtrl )
		{
			int iLength;
			TCHAR szBuffer[80];
	    TOOLTIPTEXT *pTTT = (TOOLTIPTEXT *)pnmh;

			pTTT->hinst = NULL;
			
			// Load associated string resource
			if( iLength = ::LoadString(_Module.GetResourceInstance(), idCtrl, 
																 szBuffer, sizeof(szBuffer)/sizeof(*szBuffer)) )
			{
				// Ignore everything before the newline
				LPTSTR pszText = _tcsrchr(szBuffer, '\n');
				if( pszText )
				{
					lstrcpyn( pTTT->szText, pszText + 1, iLength );	// JMO -- changed from RTL function _tcsncpy
					bHandled = true;
				}
			}
    }

		return 0;
	}
	
	//Create() which allows one to set the Toolbar bitmap and buttons too.
	HWND Create(HWND hWndParent, RECT& rcPos, LPTBBUTTON lpTBButtons,
				UINT nButtons,UINT nBmpID = nToolbarBMPID, LPCTSTR szWindowName = NULL,
				DWORD dwStyle = 0, DWORD dwExStyle = 0,
				UINT nID = nToolbarID, LPVOID lpCreateParam = NULL)
	{
		if (GetWndClassInfo().m_lpszOrigName == NULL)
			GetWndClassInfo().m_lpszOrigName = GetWndClassName();
		ATOM atom = GetWndClassInfo().Register(&m_pfnSuperWindowProc);

		dwStyle = GetWndStyle(dwStyle);
		dwExStyle = GetWndExStyle(dwExStyle);

		HWND hWnd = CWindowImplBaseT< TBase, TWinTraits >::Create(	hWndParent, 
																	rcPos, szWindowName,
																	dwStyle, dwExStyle, 
																	nID, atom, lpCreateParam);

		ATLASSERT(::IsWindow(hWnd));
		SetButtonStructSize(sizeof(TBBUTTON));

		//Set the BMP
		AddBitmap(nButtons, nBmpID);

		//Add the Buttons
		AddButtons(nButtons,lpTBButtons);
		
		//AutoSizeit
		AutoSize();
				
		return hWnd;
	}
};
typedef CToolBarImplT<CToolBarCtrl, CToolBarWinTraits> CToolBar;

/////////////////////////////////////////////////////////////////////////////
// Our StatusBar

const int nStatusBarID	= 100;//also the string res for status text eventually

template <typename TBase = CStatusBarCtrl, typename TWinTraits = CStatusBarWinTraits>
class CStatusBarImplT: public CWindowImpl<CStatusBarImplT,TBase,TWinTraits>
{
public:
	DECLARE_WND_SUPERCLASS(_T("Attila_StatusBarBase"),TBase::GetWndClassName());
	
	typedef CToolBarImplT<TBase,TWinTraits >					thisClass;
	typedef CWindowImpl<CStatusBarImplT,TBase,TWinTraits>	baseClass;
	
	BEGIN_MSG_MAP(thisClass)
	END_MSG_MAP()
	
	HWND Create(HWND hWndParent, RECT& rcPos, LPCTSTR szWindowName = NULL,
			DWORD dwStyle = 0, DWORD dwExStyle = 0,
			UINT nID = nStatusBarID, LPVOID lpCreateParam = NULL)
	{
		if (GetWndClassInfo().m_lpszOrigName == NULL)
			GetWndClassInfo().m_lpszOrigName = GetWndClassName();
		ATOM atom = GetWndClassInfo().Register(&m_pfnSuperWindowProc);

		dwStyle = GetWndStyle(dwStyle);
		dwExStyle = GetWndExStyle(dwExStyle);

		HWND hWnd = CWindowImplBaseT<TBase,TWinTraits>::Create(hWndParent, rcPos, szWindowName,dwStyle, dwExStyle, nID,atom,lpCreateParam);
		ATLASSERT(::IsWindow(hWnd));
		
		//Add the status bar text
		SetText(0, _T("Ready"));//In future this will be the resource string
		
		return hWnd;
	}
};
typedef CStatusBarImplT<CStatusBarCtrl, CStatusBarWinTraits> CStatusBar;

/////////////////////////////////////////////////////////////////////////////
// Rebar encapsulating the toolbar and the menubar

typedef CWindowImplBaseT<CReBarCtrl, CWinTraits<WS_CHILD | WS_VISIBLE| WS_CLIPSIBLINGS | WS_CLIPCHILDREN |CCS_NODIVIDER | CCS_NOPARENTALIGN | RBS_BANDBORDERS | RBS_VARHEIGHT | RBS_AUTOSIZE> > CWindowImplRebar;

template <typename TMenuBar, typename TToolBar>
class CToolPanelT: public CWindowImplRebar
{
public:
    CToolPanelT() : m_hBmp(0) {} 
	virtual ~CToolPanelT()
	{
		if(m_hBmp)
		{
			DeleteObject(m_hBmp);
			m_hBmp = 0;
		}
	}
	
BEGIN_MSG_MAP(CToolPanelT)
	MESSAGE_HANDLER(WM_SIZE, OnSize)
END_MSG_MAP()
	
	LRESULT OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) 
	{
		return 0L; 
	}
public:
	DECLARE_WND_SUPERCLASS(_T("Attila_ToolPanelBase"),GetWndClassName());
	
	HWND Create(	HWND hWndParent, RECT& rcPos, 
					LPTBBUTTON lpTBButtons,							//toolbar related
					UINT nButtons,UINT nToolbarID = 0,		//toolbar related
					UINT nMenubarID = 0,				//menubar related
					LPCTSTR szWindowName = NULL,
					DWORD dwStyle = 0, DWORD dwExStyle = 0,
					LPVOID lpCreateParam = NULL
				)
	{
		if (GetWndClassInfo().m_lpszOrigName == NULL)
			GetWndClassInfo().m_lpszOrigName = GetWndClassName();
		ATOM atom = GetWndClassInfo().Register(&m_pfnSuperWindowProc);
		
		dwStyle =	WS_CHILD | WS_VISIBLE| WS_BORDER|WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
					CCS_NODIVIDER | CCS_TOP |CCS_NOPARENTALIGN|CCS_NOMOVEY |CCS_NORESIZE|
					RBS_BANDBORDERS | RBS_VARHEIGHT ;//| RBS_AUTOSIZE;

		dwExStyle = WS_EX_LEFT | WS_EX_LTRREADING | WS_EX_RIGHTSCROLLBAR | WS_EX_TOOLWINDOW;

		HWND hWnd = CWindowImplRebar::Create(hWndParent, rcPos, szWindowName,dwStyle, dwExStyle, nMenubarID, atom, lpCreateParam);
		ATLASSERT(::IsWindow(hWnd));
		m_wndToolBar.Create(m_hWnd,CWindow::rcDefault,lpTBButtons,nButtons,nToolbarID);
		m_wndMenuBar.Create(m_hWnd,CWindow::rcDefault,nMenubarID);
		
		InitPanel();
		return hWnd;
	}

	// JR - No Longer needed.  MenuBar is tied into translation loop automatically
#if 0
	BOOL Translate(MSG* pMsg)
	{
		return m_wndMenuBar.Translate(pMsg);
	}
#endif

	void ShowToolbar(BOOL bShow)
	{
		m_wndToolBar.ShowWindow(bShow);
		if(!bShow)
		{
			DeleteBand(1); //The caller should resize things now
		}
		else if(GetBandCount() == 1)
		{
			InitPanel(TRUE);
		}
		//else NO-OP
	}
	void SetCoolBitmap(UINT nID)
	{
		HBITMAP m_hBmp = (HBITMAP)::LoadImage(_Module.GetResourceInstance(),
								MAKEINTRESOURCE(nID),
								IMAGE_BITMAP,
								0, 0,
								LR_LOADMAP3DCOLORS|LR_LOADTRANSPARENT);
		ShowCoolBitmap();
	}

protected:
	void InitPanel(BOOL bOnlyToolbar = FALSE)
	{
		REBARINFO     rbi;
		rbi.cbSize = sizeof(REBARINFO); 
		SetBarInfo(&rbi);

		REBARBANDINFO rbInfo;
			
		rbInfo.cbSize = sizeof(REBARBANDINFO);
		rbInfo.fMask = RBBIM_STYLE | RBBIM_CHILD | RBBIM_CHILDSIZE|RBBIM_TEXT ;
		rbInfo.fStyle = RBBS_BREAK| RBBS_CHILDEDGE | RBBS_GRIPPERALWAYS;
		rbInfo.lpText = _T("");
		
		int cxTB = ::GetSystemMetrics(SM_CXFULLSCREEN);//GetMenubarLength();
		
		if(!bOnlyToolbar) //menubar
		{
			rbInfo.hwndChild = m_wndMenuBar.m_hWnd;
			rbInfo.cxMinChild = cxTB;
			rbInfo.cyMinChild = 22;
			rbInfo.cx = cxTB;
			rbInfo.cxIdeal = cxTB;
			
			InsertBand(-1,&rbInfo);
		}		

		cxTB = ::GetSystemMetrics(SM_CXFULLSCREEN);//GetToolbarLength();
		
		rbInfo.hwndChild = m_wndToolBar.m_hWnd;
		rbInfo.cxMinChild = cxTB;
		rbInfo.cyMinChild = 24;
		rbInfo.cx = cxTB;
		rbInfo.cxIdeal = cxTB;
		
		//rbInfo.cyChild = rbInfo.cyMinChild;
		InsertBand(-1,&rbInfo);
	}

	void ShowCoolBitmap()
	{
		int n = GetBandCount();
		for (int i=0; i< n; i++) 
		{
			REBARBANDINFO rbbi;
			memset(&rbbi, 0, sizeof(REBARBANDINFO));
			rbbi.cbSize = sizeof(REBARBANDINFO);
			
			rbbi.fMask  = RBBIM_STYLE;
			GetBandInfo(i, &rbbi);
			rbbi.fMask  |= RBBIM_BACKGROUND;
			rbbi.fStyle |= RBBS_FIXEDBMP;
			rbbi.hbmBack = m_hBmp;
			SetBandInfo(i, &rbbi);
		}

		Invalidate();
		HWND hWndChild = ::GetWindow(m_hWnd, GW_CHILD);
		while (hWndChild != NULL) 
		{
			::InvalidateRect(hWndChild, NULL, TRUE);
			hWndChild = ::GetNextWindow(hWndChild, GW_HWNDNEXT);
		}
	}
	int GetMenubarLength()	
	{
		int nWidth = 0;
		RECT rc;

		int nCount = SendMessage(m_wndMenuBar.m_hWnd, TB_BUTTONCOUNT, 0, 0);
		for( int i = 0; i < nCount; i++ )
		{
			SendMessage(m_wndMenuBar.m_hWnd, TB_GETITEMRECT, i, (LPARAM)&rc);
			nWidth += rc.right - rc.left;
		}

		return nWidth;
	}

	int GetToolbarLength()	
	{
		int nWidth = 0;
		RECT rc;

		int nCount = SendMessage(m_wndToolBar.m_hWnd, TB_BUTTONCOUNT, 0, 0);
		for( int i = 0; i < nCount; i++ )
		{
			SendMessage(m_wndToolBar.m_hWnd, TB_GETITEMRECT, i, (LPARAM)&rc);
			nWidth += rc.right - rc.left;
		}

		return nWidth;
	}
	

public:
	TMenuBar		m_wndMenuBar;
	TToolBar		m_wndToolBar;
protected:
	HBITMAP			m_hBmp; 
};

typedef CToolPanelT<CMenuBar,CToolBar>	CToolPanel;	

/////////////////////////////////////////////////////////////////////////////
// Outlook/mail style InfoBar - under construction

template <typename TWindow = CWindow, typename TWinTraits = CInfoBarTraits>
class CInfoBarImpl: public CWindowImpl<CInfoBarImpl,TWindow,TWinTraits>
{
public:
    CInfoBarImpl(CComBSTR strTitle, UINT nResID, UINT nAlignment = DT_LEFT,
                 COLORREF clrText = RGB(255,255,255) , HFONT hFont = 0, BOOL bMask = TRUE,
                 COLORREF clrBmpMask = RGB(128,128,128)):
				m_hFont(0),
				m_nAlignment(nAlignment),
				m_clrText(clrText),
				m_strTitle(strTitle),
				m_bDeleteFont(false),
				m_hBmp(0)
	{
		if(!m_hFont)
		{
			m_hFont	= ::CreateFont(35, 0, 0, 0, FW_NORMAL, FALSE,
                                   FALSE, FALSE, ANSI_CHARSET,
                                   OUT_DEFAULT_PRECIS,
                                   CLIP_DEFAULT_PRECIS,
                                   DEFAULT_QUALITY,DEFAULT_PITCH,
                                   _T("Times New Roman"));
			m_bDeleteFont = true;
		}
		
		if (bMask)
		{		
			COLORMAP clrMap[1];

			clrMap[0].from = clrBmpMask;
			clrMap[0].to = ::GetSysColor(COLOR_APPWORKSPACE);

			m_hBmp = CreateMappedBitmap(_Module.GetResourceInstance(),
                                        nResID, 0, clrMap, 1);
		}
		else
		{
			m_hBmp = LoadBitmap(_Module.GetResourceInstance(),MAKEINTRESOURCE(nResID));
		}
	}

    ~CInfoBarImpl()
	{
		if(m_hFont && m_bDeleteFont)
		{
			::DeleteObject(m_hFont);
			m_hFont = NULL;
		}
		if(m_hBmp)
		{
			::DeleteObject(m_hBmp); m_hBmp = 0;
		}
	}

	DECLARE_WND_CLASS_EX(_T("InfoBar"), CS_DBLCLKS, COLOR_APPWORKSPACE)
	
	typedef CWindowImpl<CInfoBarImpl,TWindow,TWinTraits> baseclass;
	typedef CInfoBarImpl<TWindow,TWinTraits> thisclass;

public:
	BEGIN_MSG_MAP(thisclass)
		MESSAGE_HANDLER(WM_PAINT, OnPaint)
	END_MSG_MAP()
	
	LRESULT OnPaint(UINT, WPARAM, LPARAM, BOOL&)
    {
		PAINTSTRUCT ps;
        HDC         hdc = BeginPaint(&ps);
        
		//Bk/Fr gnd
		::SetTextColor(hdc, RGB(255,255,255) );
		::SetBkColor(hdc, ::GetSysColor(COLOR_APPWORKSPACE));
		
		//Font
		HFONT hFontOld = (HFONT)::SelectObject(hdc,m_hFont);		
		//Textout
		RECT    rect; GetClientRect(&rect);
		USES_CONVERSION;
        RECT  textRect;
        textRect = rect;
        textRect.right = (rect.right - rect.left - 40);
        ::DrawTextEx(hdc, OLE2T(m_strTitle), m_strTitle.Length(), &textRect, DT_END_ELLIPSIS, NULL);
        //cleanup
		::SelectObject(hdc,hFontOld);		

		//Bitmap
		HDC hDCMem =  ::CreateCompatibleDC(hdc);
		HBITMAP hBmpOld = (HBITMAP)::SelectObject(hDCMem,m_hBmp);
		
		BITMAP bmp;
		::GetObject(m_hBmp, sizeof(BITMAP), &bmp);
		SIZE size = { bmp.bmWidth,bmp.bmHeight };
				
		::StretchBlt(hdc, (rect.right - rect.left - 40),rect.top + 2, 32, 32, hDCMem, 0, 0, size.cx,size.cy,SRCCOPY);
		
		//cleanup
		::SelectObject(hDCMem,hBmpOld);
		EndPaint(&ps);
		
		return 0;
	}
private:
	bool		m_bDeleteFont;
	HFONT		m_hFont;
	HBITMAP		m_hBmp;
	UINT		m_nAlignment;
	COLORREF	m_clrText;
	CComBSTR	m_strTitle;
};

typedef class CInfoBarImpl<CWindow,CInofBarWinTraits> CInfoBar;

}   // namespace Attila

#ifndef _ATTILA_NOUSE_NAMESPACE
using namespace Attila;
#endif

#endif // INC_ATTILABARS
