// AttilaView.h
/////////////////////////////////////////////////////////////////////////////
// This is part of the Attila library.
// Copyright (c) 1999, Attila contributors (see ReadMe.htm).
// All rights reserved. No warranties extended.
/////////////////////////////////////////////////////////////////////////////

#pragma once
#ifndef INC_ATTLIAVIEW
#define INC_ATTLIAVIEW

#include "AttilaCommon.h"

namespace Attila
{

/////////////////////////////////////////////////////////////////////////////
// Our NOOP View (for now)

template <typename TDerived, typename TWindow= CWindow, typename TWinTraits = CViewWinTraits>
class ATL_NO_VTABLE CViewWindowImpl: public CWindowImpl<CViewWindowImpl,TWindow,TWinTraits>
{
public:
	typedef CViewWindowImpl<TDerived, TWindow, TWinTraits >	thisClass;
	typedef CWindowImpl<CViewWindowImpl,TWindow,TWinTraits>	baseClass;

	BEGIN_MSG_MAP(thisClass)
	END_MSG_MAP()
};

/////////////////////////////////////////////////////////////////////////////
// Our Dialog View w/o many frills  
// Tis Modeless - 
//
// We need a CMsgTranslator here for routing kebd messages across AX and
// dialog controls IsDialogMessage() will take care of only normal ones.

template <typename TDerived,typename TBase = CWindow>
class ATL_NO_VTABLE CViewDialogImpl: public CAxDialogImpl<CViewDialogImpl,TBase>,public CMsgTranslator
{
public:
	typedef CViewDialogImpl<TDerived,TBase>					thisClass;
	typedef CAxDialogImpl<CViewDialogImpl,TBase>				theBase;

	BEGIN_MSG_MAP(thisClass)
		MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
	END_MSG_MAP()

	LRESULT OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		return 1;  // system will set the focus
	}
	
public:
	// ATL's AX Create() *as is* for modeless dialogs
	HWND Create(HWND hWndParent, LPARAM dwInitParam = NULL)
	{
		ATLASSERT(m_hWnd == NULL);
		_Module.AddCreateWndData(&m_thunk.cd, (CDialogImplBaseT< TBase >*)this);

		HWND hWnd = AtlAxCreateDialog(_Module.GetResourceInstance(), MAKEINTRESOURCE(TDerived::IDD),
					hWndParent, (DLGPROC)StartDialogProc, dwInitParam);
		ATLASSERT(m_hWnd == hWnd);
		return hWnd;
	}

	//This is blatant copy ATL's PreTranslateAccelerator() minus its *hRet*
	BOOL Translate(const MSG& rMsg,HACCEL hAccel)
	{
		if ((rMsg.message < WM_KEYFIRST || rMsg.message > WM_KEYLAST) &&
		   (rMsg.message < WM_MOUSEFIRST || rMsg.message > WM_MOUSELAST))
			return FALSE;
		// find a direct child of the dialog from the window that has focus
		HWND hWndCtl = ::GetFocus();
		if (IsChild(hWndCtl) && ::GetParent(hWndCtl) != m_hWnd)
		{
			do
			{
				hWndCtl = ::GetParent(hWndCtl);
			}
			while (::GetParent(hWndCtl) != m_hWnd);
		}
		// give controls a chance to translate this message
		if (::SendMessage(hWndCtl, WM_FORWARDMSG, 0, (LPARAM)&rMsg) == 1)
			return TRUE;

		// special handling for keyboard messages
		switch(rMsg.message)
		{
		case WM_CHAR:
			if(::SendMessage(rMsg.hwnd, WM_GETDLGCODE, 0, 0L) == 0)	// no dlgcode, possibly an ActiveX control
				return FALSE;	// let the container process this
			break;
		case WM_KEYDOWN:
			switch(LOWORD(rMsg.wParam))
			{
			case VK_EXECUTE:
			case VK_RETURN:
			case VK_ESCAPE:
			case VK_CANCEL:
				// we don't want to handle these, let the container do it
				return FALSE;
			}
			break;
		}

		return IsDialogMessage(const_cast<MSG *>(&rMsg));
	}
};

}   // namespace Attila

#ifndef _ATTILA_NOUSE_NAMESPACE
using namespace Attila;
#endif

#endif  // INC_ATTLIAVIEW
